==> ideaIC-213.7172.25-sources.jar: com/android/tools/idea/layoutinspector/properties/InspectorPropertiesView.kt <==
/*
 * Copyright (C) 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.layoutinspector.properties

import com.android.tools.property.panel.api.ControlType
import com.android.tools.property.panel.api.ControlTypeProvider
import com.android.tools.property.panel.api.EnumSupport
import com.android.tools.property.panel.api.EnumSupportProvider
import com.android.tools.property.panel.api.PropertiesView
import com.android.tools.property.panel.api.Watermark
import com.android.tools.property.ptable2.PTableItem
import org.jetbrains.android.formatter.AttributeComparator
import com.android.tools.idea.layoutinspector.properties.PropertyType as Type

private const val VIEW_NAME = "LayoutInspectorPropertyEditor"
private const val WATERMARK_MESSAGE = "No view selected."
private const val WATERMARK_ACTION_MESSAGE = "Select a view in the Component Tree."

val TEXT_RESOURCE_EDITOR = ControlType.CUSTOM_EDITOR_1
val COLOR_RESOURCE_EDITOR = ControlType.CUSTOM_EDITOR_2

/**
 * Comparator that is sorting [PTableItem] in Android sorting order.
 * This implies layout attributes first and layout_width before layout_height.
 */
val androidSortOrder: Comparator<PTableItem> = AttributeComparator { it.name }

class InspectorPropertiesView(model: InspectorPropertiesModel) : PropertiesView<InspectorPropertyItem>(VIEW_NAME, model) {

  private val enumSupportProvider = object : EnumSupportProvider<InspectorPropertyItem> {
    // TODO: Make this a 1 liner
    override fun invoke(property: InspectorPropertyItem): EnumSupport? = null
  }

  private val controlTypeProvider = object : ControlTypeProvider<InspectorPropertyItem> {
    override fun invoke(property: InspectorPropertyItem): ControlType {
      return when (property.type) {
        Type.DRAWABLE,
        Type.COLOR -> if (property.needsResolutionEditor) COLOR_RESOURCE_EDITOR else ControlType.COLOR_EDITOR
        Type.LAMBDA,
        Type.FUNCTION_REFERENCE,
        Type.SHOW_MORE_LINK -> ControlType.LINK_EDITOR
        else -> if (property.needsResolutionEditor) TEXT_RESOURCE_EDITOR else ControlType.TEXT_EDITOR
      }
    }
  }

  init {
    watermark = Watermark(WATERMARK_MESSAGE, WATERMARK_ACTION_MESSAGE, "")
    main.builders.add(SelectedViewBuilder)
    val attributeSections = setOf(PropertySection.DEFAULT, PropertySection.DECLARED, PropertySection.LAYOUT)
    val tab = addTab("")
    tab.builders.add(DimensionBuilder)
    tab.builders.add(InspectorTableBuilder("Declared Attributes", { it.section == PropertySection.DECLARED },
                                           model, enumSupportProvider, controlTypeProvider))
    tab.builders.add(InspectorTableBuilder("Layout", { it.section == PropertySection.LAYOUT },
                                           model, enumSupportProvider, controlTypeProvider, androidSortOrder))
    tab.builders.add(InspectorTableBuilder("All Attributes", { it.section in attributeSections },
                                           model, enumSupportProvider, controlTypeProvider, searchable = true))
    tab.builders.add(InspectorTableBuilder("Parameters", { it.section == PropertySection.PARAMETERS },
                                           model, enumSupportProvider, controlTypeProvider, searchable = true))
    tab.builders.add(InspectorTableBuilder("Merged Semantics", { it.section == PropertySection.MERGED },
                                           model, enumSupportProvider, controlTypeProvider, searchable = true))
    tab.builders.add(InspectorTableBuilder("Semantics", { it.section == PropertySection.UNMERGED },
                                           model, enumSupportProvider, controlTypeProvider, searchable = true))
  }
}

==> ideaIC-2019.3.5-sources.jar: com/android/tools/idea/layoutinspector/properties/InspectorPropertiesView.kt <==
/*
 * Copyright (C) 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.layoutinspector.properties

import com.android.tools.layoutinspector.proto.LayoutInspectorProto.Property.Type
import com.android.tools.property.panel.api.ControlType
import com.android.tools.property.panel.api.ControlTypeProvider
import com.android.tools.property.panel.api.EditorProvider
import com.android.tools.property.panel.api.EnumSupport
import com.android.tools.property.panel.api.EnumSupportProvider
import com.android.tools.property.panel.api.PropertiesView
import com.android.tools.property.panel.api.Watermark

private const val VIEW_NAME = "LayoutInspectorPropertyEditor"
private const val WATERMARK_MESSAGE = "No view selected."
private const val WATERMARK_ACTION_MESSAGE = "Select a view in the Component Tree."

class InspectorPropertiesView(model: InspectorPropertiesModel) : PropertiesView<InspectorPropertyItem>(VIEW_NAME, model) {

  private val enumSupportProvider = object : EnumSupportProvider<InspectorPropertyItem> {
    // TODO: Make this a 1 liner
    override fun invoke(property: InspectorPropertyItem): EnumSupport? = null
  }

  private val controlTypeProvider = object : ControlTypeProvider<InspectorPropertyItem> {
    override fun invoke(property: InspectorPropertyItem): ControlType =
      when (property.type) {
        Type.COLOR -> ControlType.COLOR_EDITOR
        else -> ControlType.TEXT_EDITOR
      }
  }

  private val editorProvider = EditorProvider.create(enumSupportProvider, controlTypeProvider)

  init {
    watermark = Watermark(WATERMARK_MESSAGE, WATERMARK_ACTION_MESSAGE, "")
    val tab = addTab("")
    tab.builders.add(InspectorTableBuilder("Declared Attributes", { it.isDeclared }, model, controlTypeProvider, editorProvider))
    tab.builders.add(InspectorTableBuilder("All Attributes", { true }, model, controlTypeProvider, editorProvider))
  }
}
